@@ -34,6 +34,8 @@ from random import randint
 from time import sleep
 from typing import Mapping, Sequence, Union, Tuple
 
+from qalx_orcaflex.core import QalxSmartStaticsFailed
+
 try:  # we build the docs on linux and this won't work...
     import OrcFxAPI
 except (ImportError, ValueError):
@@ -62,6 +64,7 @@ from qalx_orcaflex.helpers import (
     get_tags,
 )
 from qalx_orcaflex.helpers.smart_statics import solved_model
+from qalx_orcaflex.video.extract import video_extractor
 
 
 def initialise_orcaflex_bot(qalx):
@@ -212,9 +215,20 @@ def process_orcaflex(job: QalxJob) -> None:
 
         try:
             start = time.time()  # we time how long statics takes
-            solved_model(
-                model, job.options.max_search_attempts
-            )  # solves with search if required
+            try:
+                solved_model(
+                    model, job.options.max_search_attempts
+                )  # solves with search if required
+            except QalxSmartStaticsFailed as err:
+                fn = f"{job.entity['meta']['case_name']}_FAILED_SS.dat"
+                ff = job.session.item.add(
+                    source=BytesIO(model.SaveDataMem()),
+                    file_name=fn,
+                    meta={"_class": "orcaflex.failed_statics"},
+                )
+                job.entity["items"]["failed_statics"] = ff
+                job.save_entity()
+                raise err
             end = time.time()
             statics["time_to_solve"] = end - start
             statics["solves"] = True
@@ -481,6 +495,9 @@ def process_orcaflex(job: QalxJob) -> None:
     if ofx.model_views or get_tags(model, lambda t: t.startswith("mv__"), True):
         extract_model_views(model)
 
+    if ofx.model_videos:
+        video_extractor.extract_model_videos(job=job, ofx=ofx, model=model)
+
     if (
         str(model.state) == "SimulationStoppedUnstable"
     ):  # if the simulation was unstable then let us know
