@@ -1,14 +1,14 @@
 import concurrent.futures
 from copy import deepcopy
 from dataclasses import asdict
-from io import BytesIO
 from os import PathLike
 from pathlib import Path
+from tempfile import TemporaryDirectory
 from typing import Union, Sequence, Mapping
 
 import pyqalx
 from OrcFxAPI import Model
-from pyqalx import QalxSession, Item, Group
+from pyqalx import QalxSession, Item, Group, Set
 
 from qalx_orcaflex import data_models
 from qalx_orcaflex.helpers import (
@@ -16,6 +16,7 @@ from qalx_orcaflex.helpers import (
     required_results_from_yaml,
     clean_set_key,
 )
+from qalx_orcaflex.video.helpers import unpack_model_videos, check_sim_file_type
 
 LOAD_CASE_INFO = data_models.LoadCaseInfo()
 
@@ -27,12 +28,22 @@ class QalxOrcaFlexError(Exception):
         pass
 
 
+class QalxSmartStaticsError(QalxOrcaFlexError):
+    """Error related to the smart statics utility."""
+
+
+class QalxSmartStaticsFailed(QalxSmartStaticsError):
+    """We tried to do smart statics and it didn't work."""
+
+
 class BaseSource:
     """base class for batch sources.
 
-    Any child class must implement a `to_jobs` method which takes a base_job argument
-    and returns a generator of jobs
+    Any child class must implement a `to_jobs` method which takes a base_job and
+    temp_folder argument and returns a generator of jobs
 
+    the temp_folder will be automatically deleted by the batch manager after the
+    batch has been submitted.
     """
 
     def __init__(self, *args, **kwargs):
@@ -52,7 +63,7 @@ class BaseSource:
         data_file["meta"]["data_file_uri"] = dfp.as_uri()
         return dfp
 
-    def to_jobs(self, base_job: Mapping) -> Mapping:
+    def to_jobs(self, base_job: Mapping, temp_folder: Path) -> Mapping:
         raise NotImplementedError(
             "This method needs to be defined in all BaseSource child classes"
         )
@@ -69,7 +80,7 @@ class DataFileItemsSource(BaseSource):
         super(DataFileItemsSource, self).__init__(items)
         self.items = items
 
-    def to_jobs(self, base_job: Mapping) -> Mapping:
+    def to_jobs(self, base_job: Mapping, temp_folder: Path) -> Mapping:
         for dfi in self.items:
             job = self._update_copy(
                 base_job,
@@ -89,18 +100,17 @@ class ModelSource(BaseSource):
         :param name: the name of the model, used as the file name to save back later.
         """
         super(ModelSource, self).__init__(model, name)
-        self.model_bytes = model.SaveDataMem()
+        self.model = model
         self.name = name
         self._clean_name = clean_set_key(name)
 
-    def to_jobs(self, base_job: Mapping) -> Mapping:
+    def to_jobs(self, base_job: Mapping, temp_folder: Path) -> Mapping:
+        filename = f"{self.name}.dat"
+        model_path = temp_folder.joinpath(filename)
+        self.model.SaveData(model_path)
         data_file = {
-            "source": BytesIO(self.model_bytes),
-            "meta": {
-                "_class": "orcaflex.job.data_file",
-                "data_file_name": f"{self.name}.dat",
-            },
-            "file_name": f"{self.name}.dat",
+            "source": model_path,
+            "meta": {"_class": "orcaflex.job.data_file", "data_file_name": filename},
         }
         job = self._update_copy(
             base_job, {"data_file": data_file, "case_name": self._clean_name}
@@ -149,7 +159,7 @@ class DirectorySource(BaseSource):
         self.skip_fails = skip_fails
         self.recursive = recursive
 
-    def to_jobs(self, base_job: Mapping) -> Mapping:
+    def to_jobs(self, base_job: Mapping, temp_folder: Path) -> Mapping:
         for dfp in data_file_paths(
             self.directory, self.include_yaml, self.skip_fails, self.recursive
         ):
@@ -167,12 +177,13 @@ class DirectorySource(BaseSource):
 class OrcaFlexBatchManager:
     """manager class for collecting batch info"""
 
-    def __init__(self):
+    def __init__(self, temp_folder):
         """instance is returned by :ref:OrcaFlexBatch as context manager
 
         This collects all the sources for the jobs added to the batch.
         """
         self.orcaflex_jobs = []
+        self._temp_folder = Path(temp_folder)
 
     def add(
         self,
@@ -182,6 +193,7 @@ class OrcaFlexBatchManager:
             PathLike, Sequence[Union[data_models.TimeHistory, data_models.RangeGraph]]
         ] = (),
         model_views: Sequence[data_models.ModelView] = (),
+        model_videos: Sequence[data_models.VideoSpecification] = None,
         load_case_info: data_models.LoadCaseInfo = LOAD_CASE_INFO,
     ):
         """add data files to a batch based on a source and extra options.
@@ -194,6 +206,7 @@ class OrcaFlexBatchManager:
             (a mix of both is allowed) or a path to a yaml file with the correct
             Results structure.
         :param model_views: an iterator containing :ref:data_models.ModelView
+        :param model_videos: An list of VideoSpecification instances
         :param load_case_info: information about the load case to be used during
             presentation of results
             see :ref:data_models.LoadCaseInfo
@@ -206,8 +219,9 @@ class OrcaFlexBatchManager:
             "required_results": required_results,
             "load_case_info": load_case_info,
             "model_views": model_views,
+            "model_videos": model_videos,
         }
-        for job in source.to_jobs(base_job):
+        for job in source.to_jobs(base_job, self._temp_folder):
             self.orcaflex_jobs.append(job)
 
 
@@ -240,6 +254,7 @@ class OrcaFlexBatch:
         self.name = name
         self.session = session
         self.batch_options = batch_options
+        self.temp_dir = TemporaryDirectory()
         self.dry_run = dry_run
         standard_meta = {
             "_class": "orcaflex.batch",
@@ -363,7 +378,15 @@ class OrcaFlexBatch:
                     "case_guid": self.built_sets[case_name].guid,
                 },
             )
-
+        if job.get("model_videos"):
+            model_videos = unpack_model_videos(job)
+            job_items["model_videos"] = dict(
+                data=model_videos,
+                meta={
+                    "_class": "orcaflex.job.model_videos",
+                    "case_guid": self.built_sets[case_name].guid,
+                },
+            )
         job_item_many = self.session.item.add_many(
             list(job_items.values())
         )  # add all the items
@@ -432,12 +455,15 @@ class OrcaFlexBatch:
         Group.add_to_queue(payload=self.group_batch, queue=batch_queue)
 
     def __enter__(self):
-        self.batch_manager = OrcaFlexBatchManager()
+        self.temp_dir.__enter__()
+        self.batch_manager = OrcaFlexBatchManager(self.temp_dir.name)
         return self.batch_manager
 
     def __exit__(self, *args):
-        self._build()
-        self._submit()
+        if not args[1]:  # if there is an error this argument will be True
+            self._build()
+            self._submit()
+        self.temp_dir.__exit__(*args)
 
 
 class QalxOrcaFlex(pyqalx.QalxSession):
@@ -510,6 +536,58 @@ class QalxOrcaFlex(pyqalx.QalxSession):
                     save_name = f"{case_name}-{view_file['file']['name']}"
                     view_file.save_file_to_disk(save_dir, filename=save_name)
 
+    def send_to_video_sim_queue(self, queue_name, ofx_job):
+        if ofx_job.sim_file is None and ofx_job.sim_file_path is None:
+            raise AttributeError("The job must have a sim_file or sim_file_path")
+        if ofx_job.sim_file is None:
+            # If the file does not exist, then add it to qalx
+            check_sim_file_type(ofx_job.sim_file_path)
+            ofx_job.sim_file = self.item.add(
+                source=ofx_job.sim_file_path, meta={"_class": "orcaflex.job.sim_file"}
+            )
+        else:
+            check_sim_file_type(ofx_job.sim_file["file"]["name"])
+        model_videos = unpack_model_videos(ofx_job.__dict__)
+        job_items = {
+            "model_videos": dict(
+                data=model_videos, meta={"_class": "orcaflex.job.model_videos"}
+            ),
+            "job_options": dict(
+                data=asdict(ofx_job.job_options),
+                meta={"_class": "orcaflex.job.options"},
+            ),
+        }
+        job_item_many = self.item.add_many(list(job_items.values()))
+        job_items_zipped = dict(zip(job_items.keys(), job_item_many["items"]))
+        job_items_zipped["sim_file"] = ofx_job.sim_file
+        video_job = self.set.add(
+            items=job_items_zipped,
+            meta={"_class": "orcaflex.job.video_extraction_from_sim"},
+        )
+        video_queue = self.queue.get_or_create(queue_name)
+        Set.add_to_queue(payload=video_job, queue=video_queue)
+        return video_job.guid
+
+    def save_job_videos(self, job, save_dir, case_name=""):
+        if not isinstance(job, Set):
+            job = self.set.get(job)
+        if job["items"].get("saved_videos"):
+            if not isinstance(job["items"].get("saved_videos"), Mapping):
+                videos = self.item.get(job["items"].get("saved_videos"))["data"]
+            else:
+                videos = job["items"].get("saved_videos")["data"]
+            for _, video_guid in videos.items():
+                video_file = self.item.get(video_guid)
+                save_name = f"{video_file['file']['name']}"
+                if case_name:
+                    save_name = f"{case_name}-" + save_name
+                video_file.save_file_to_disk(save_dir, filename=save_name)
+
+    def save_batch_videos(self, batch_name: str, save_dir: PathLike):
+        batch = self.get_batch(batch_name)
+        for case_name, job in batch["sets"].items():
+            self.save_job_videos(job, save_dir, case_name)
+
     def _get_progress(self, job):
         if job["items"].get("progress"):
             if not job["meta"].get("processing_complete"):
