@@ -5,7 +5,7 @@ Models
 This is set of `dataclass` classes that help define elements of the system.
 
 These data_models are used to be able to easily construct data structures that are
-requried for the OrcaFlex simulation processing system to work.
+required for the OrcaFlex simulation processing system to work.
 
 Some of these data_models can be used at the definition stage where you are required to
 set some options or settings, provide source data or pre-specify some thing that you
@@ -22,6 +22,8 @@ from typing import List, Mapping, Any, Tuple, Sequence
 import OrcFxAPI
 from pyqalx.core.entities import Item
 
+from ..video import DEFAULT_WIDTH, DEFAULT_HEIGHT
+
 OE_LINE_PT = {
     "End A": OrcFxAPI.ptEndA,
     "End B": OrcFxAPI.ptEndB,
@@ -858,6 +860,27 @@ class ModelView:
         return vp
 
 
+@dataclass
+class VideoSpecification:
+    """
+    Video parameters for extracting a video from an orcaflex model
+
+    :param start_time [float]: The start time in the model to start recording
+    :param end_time [float]: The end time in the model to stop the recording
+    :param width [float]: The width of the video in pixels
+    :param height [float]: The height of the video in pixels
+    :param codec [str]: The codec of the video. Accepted values: "MRLE", "XVID"
+    :param model_views [List]: list of ref:ModelView
+    """
+
+    start_time: float
+    end_time: float
+    width: float = DEFAULT_WIDTH
+    height: float = DEFAULT_HEIGHT
+    codec: str = None
+    model_views: List[ModelView] = None
+
+
 @dataclass
 class JobProgress:
     """progress of the job
@@ -912,7 +935,7 @@ class OrcaFlexJob:
     additional_files: Item = None  # TODO: Implement and document additional files
     model_views: List[ModelView] = None
     saved_views: List[Item] = None
-    model_videos: List[ModelView] = None  # TODO: Implement and document video
+    model_videos: List[VideoSpecification] = None
     saved_videos: List[Item] = None
     progress: Item = None
     warnings: Item = None
